import re
from functools import reduce, partial as prtl

def compose(*funcs):
    """
    def f(x): pass
    def g(x): pass
    compose(g, f)(x) == g(f(x))
    """
    return reduce(
        lambda f, g: lambda x: f(g(x)), 
        tuple(reversed(funcs)), 
        lambda x: x
    )

def pipe(obj, *funcs):
    """
    def f(x): pass
    def g(x): pass
    pipe(x, f, g) = g(f(x))
    """
    func = compose(*funcs)
    return func(obj)

def valmap(func, d):
    return {k: func(v) for k, v in d.items()}

def keymap(func, d):
    return {func(k): v for k, v in d.items()}

def merge(base_d, *d):
    if len(d) == 0: return base_d
    if len(d) > 0: 
        return reduce(lambda acc, x: {**(acc or {}), **x}, d)

def seq(*values):
    assert len(values) > 0, "Must provide at list 1 value"
    if len(values) > 1:
        return tuple(*values)
    elif isinstance(values[0], (list, tuple)):
        return values[0]
    elif isinstance(values[0], dict):
        return tuple(k for k in values[0])
    elif isinstance(values[0], str):
        return [values[0]]
    else:
        values[0]
        

invalid = "[\s_.-]"
pipe(
    # raw text
    " .CamelCase that-isOK  ",
    # decamel
    prtl(re.sub, "(?<=[a-z])(?P<match>[A-Z])", "_\g<match>"),
    # trim
    prtl(re.sub, f"^{invalid}+|{invalid}+$", ""),
    # replace invalid with "_"
    prtl(re.sub, f"{invalid}", "_"),    
    # lower case
    str.lower,
    # capitalize case
    prtl(re.sub, "case", "CASE"),
)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jedi_toolz']

package_data = \
{'': ['*']}

install_requires = \
['decorator>=4.4.2,<5.0.0',
 'openpyxl>=3.0.3,<4.0.0',
 'pydomo>=0.2.3,<0.3.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'tabulate>=0.8.7,<0.9.0',
 'toolz>=0.10.0,<0.11.0']

setup_kwargs = {
    'name': 'jedi-toolz',
    'version': '0.0.17',
    'description': 'A collection of utilities to simplify a data workflow using pandas, Excel, DOMO, and other tools.',
    'long_description': None,
    'author': 'JediHero',
    'author_email': 'hansen.rusty@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/JediHero/jedi_toolz',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1',
}


setup(**setup_kwargs)
